/*
 * SN74HC165N_shift_reg
 *
 * Program to shift in the bit values from a SN74HC165N 8-bit
 * parallel-in/serial-out shift register.
 *
 * This sketch demonstrates reading in 16 digital states from a
 * pair of daisy-chained SN74HC165N shift registers while using
 * only 4 digital pins on the Arduino.
 *
 * You can daisy-chain these chips by connecting the serial-out
 * (Q7 pin) on one shift register to the serial-in (Ds pin) of
 * the other.
 * 
 * Of course you can daisy chain as many as you like while still
 * using only 4 Arduino pins (though you would have to process
 * them 4 at a time into separate unsigned long variables).
 * 
*/

#include "Arduino.h"
#include "GBController.h"

#define NUMBER_OF_SHIFT_CHIPS   1
#define DATA_WIDTH	NUMBER_OF_SHIFT_CHIPS * 8
#define PULSE_WIDTH_USEC   5
#define POLL_DELAY_MSEC   1

int ControlLoadPin        = A0;  // Connects to Parallel load pin the 165
int ControlCEPin  	= A1;  // Connects to Clock Enable pin the 165
int ControlDataPin         = 12; // Connects to the Q7 pin the 165
int ControlClockPin        = A2; // Connects to the Clock pin the 165

BYTES_VAL_T pinValues;
BYTES_VAL_T oldPinValues;

/* This function is essentially a "shift-in" routine reading the
 * serial Data from the shift register chips and representing
 * the state of those pins in an unsigned integer (or long).
*/
BYTES_VAL_T GBController::read_shift_regs()
{
    long bitVal;
    BYTES_VAL_T bytesVal = 0;
 
    /* Trigger a parallel Load to latch the state of the data lines,
    */
    digitalWrite(ControlCEPin, HIGH);
    digitalWrite(ControlLoadPin, LOW);
    delayMicroseconds(PULSE_WIDTH_USEC);
    digitalWrite(ControlLoadPin, HIGH);
    digitalWrite(ControlCEPin, LOW);

    /* Loop to read each bit value from the serial out line
     * of the SN74HC165N.
    */
    for(int i = 0; i < DATA_WIDTH; i++)
    {
        bitVal = digitalRead(ControlDataPin);

        /* Set the corresponding bit in bytesVal.
        */
        bytesVal |= (bitVal << ((DATA_WIDTH-1) - i));

        /* Pulse the Clock (rising edge shifts the next bit).
        */
        digitalWrite(ControlClockPin, HIGH);
        delayMicroseconds(PULSE_WIDTH_USEC);
        digitalWrite(ControlClockPin, LOW);
    }
    Serial.write((byte)bytesVal);
    return(bytesVal);
}

/* Dump the list of zones along with their current status.
*/
/*void GBController::display_pin_values()
{
    Serial.print("Pin States:\r\n");

    for(int i = 0; i < DATA_WIDTH; i++)
    {
        Serial.print("  Pin-");
        Serial.print(i);
        Serial.print(": ");

        if((pinValues >> i) & 1)
            Serial.print("HIGH");
        else
            Serial.print("LOW");

        Serial.print("\r\n");
    }

    Serial.print("\r\n");
    oldPinValues = pinValues;
    Serial.print(oldPinValues);
    Serial.print(pinValues);
}
*/

void GBController::display_pin_values()
{
    //unsigned int buttonState[9];
    for(int i = 0; i < DATA_WIDTH; i++)
    {
        if((pinValues >> i) & 1)
            Serial.print(1);
	    //buttonState += 1;
        else
            Serial.print(0);
	    //buttonState += 0;
    }
    oldPinValues = pinValues;
}

GBController::GBController()
{
    //Serial.begin(9600);

    /* Initialize our digital pins...
    */
    pinMode(ControlLoadPin, OUTPUT);
    pinMode(ControlCEPin, OUTPUT);
    pinMode(ControlClockPin, OUTPUT);
    pinMode(ControlDataPin, INPUT);

    digitalWrite(ControlClockPin, LOW);
    digitalWrite(ControlLoadPin, HIGH);

    /* Read in and display the pin states at startup.
    */
    pinValues = read_shift_regs();
    //display_pin_values();
    //Serial.print(oldPinValues);
}

void GBController::ControllerUpdate()
{
    /* Read the state of all zones.
    */
    pinValues = read_shift_regs();
    //Serial.print(pinValues);
    display_pin_values();

    /* If there was a change in state, display which ones changed.
    */
    if(pinValues != oldPinValues)
    {
        //Serial.print("*Pin value change detected*\r\n");
        //display_pin_values();
        //oldPinValues = pinValues;
    }

    delay(POLL_DELAY_MSEC);
}
